package com.bcxin.identity.domains.entities;

import com.bcxin.Infrastructures.exceptions.ArgumentTenantException;
import com.bcxin.Infrastructures.exceptions.BadTenantException;
import com.bcxin.identity.domains.components.PasswordEncoder;
import com.bcxin.identity.domains.enums.AlgorithmType;
import com.bcxin.identity.domains.enums.EventAction;
import com.bcxin.identity.domains.enums.PrincipalStatus;
import com.bcxin.identity.domains.events.IdentityUserDomainEvent;
import com.bcxin.identity.domains.exceptions.IdentityNotMatchedException;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.StringUtils;

import javax.persistence.*;
import java.sql.Timestamp;
import java.time.Instant;

@Getter
@Setter(AccessLevel.PROTECTED)
@Entity
@Table(name = "identity_userpassword",
        uniqueConstraints = @UniqueConstraint(name = "identity_username_password_username",columnNames = "user_name"))
public class UserNamePasswordPrincipalEntity extends PrincipalAbstract {
    @Column(name = "user_name", nullable = false,length = 50)
    private String userName;
    @Column(name = "password", nullable = false, length = 200)
    private String password;

    @Enumerated(EnumType.ORDINAL)
    @Column(nullable = false,name = "algorithm_type")
    private AlgorithmType algorithmType;

    protected UserNamePasswordPrincipalEntity(){}

    public static UserNamePasswordPrincipalEntity create(
            IdentityUserEntity identityUser,
            String id,
            String userName,
            AlgorithmType algorithmType,
            String password) {
        UserNamePasswordPrincipalEntity account =
                new UserNamePasswordPrincipalEntity();
        account.setIdentityUser(identityUser);
        account.setUserName(userName);
        account.setPassword(password);
        account.setId(id);
        account.setAlgorithmType(algorithmType);
        account.setStatus(PrincipalStatus.ACTIVATE);

        return account;
    }

    public void changePassword(PasswordEncoder passwordEncoder,
                               String oldPassword, String newPassword,
                               String confirmPassword) {
        if (!passwordEncoder.isMatched(this.getPassword(), oldPassword)) {
            throw new IdentityNotMatchedException();
        }

        if (!StringUtils.equals(newPassword, confirmPassword)) {
            throw new BadTenantException("新密码与确认密码不一致!");
        }

        this.setAlgorithmType(AlgorithmType.SM2);
        this.setPassword(passwordEncoder.encode(this.getAlgorithmType(), newPassword));

        this.recordEvent(
                IdentityUserDomainEvent.create(EventAction.CHANGED_PWD,
                        "更改密码", this.getIdentityUser(), this));
    }

    public void changeUserName(String userName) {
        if (!org.springframework.util.StringUtils.hasLength(userName)) {
            throw new ArgumentTenantException("用户名不能为空");
        }

        this.setUserName(userName);
        this.setLastUpdatedTime(Timestamp.from(Instant.now()));
    }

    public void resetPassword(PasswordEncoder passwordEncoder,
                              String newPassword,
                               String confirmPassword) {
        if (!StringUtils.equals(newPassword, confirmPassword)) {
            throw new BadTenantException("新密码与确认密码不一致!");
        }

        this.setAlgorithmType(AlgorithmType.SM2);
        this.setPassword(passwordEncoder.encode(this.getAlgorithmType(), newPassword));

        this.recordEvent(
                IdentityUserDomainEvent.create(EventAction.CHANGED_PWD,
                        "更改密码", this.getIdentityUser(), this));
    }
}
